/*
 The MIT License (MIT)

 Copyright (c) 2016 Swiftification

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 */

import Foundation

public extension Dispatch {

  static func async(closure: DispatchClosure) -> Dispatch {
    return async(Queue.main, closure: closure)
  }

  public func async(closure: DispatchClosure) -> Dispatch {
    return async(Queue.main, closure: closure)
  }

  static func asyncBackground(closure: DispatchClosure) -> Dispatch {
    return async(Queue.globalBackground, closure: closure)
  }

  func asyncBackground(closure: DispatchClosure) -> Dispatch {
    return async(Queue.globalBackground, closure: closure)
  }

  static func asyncUtility(closure: DispatchClosure) -> Dispatch {
    return async(Queue.globalUtility, closure: closure)
  }

  func asyncUtility(closure: DispatchClosure) -> Dispatch {
    return async(Queue.globalUtility, closure: closure)
  }

  static func asyncUserInitiated(closure: DispatchClosure) -> Dispatch {
    return async(Queue.globalUserInitiated, closure: closure)
  }

  func asyncUserInitiated(closure: DispatchClosure) -> Dispatch {
    return async(Queue.globalUserInitiated, closure: closure)
  }

  static func asyncUserInteractive(closure: DispatchClosure) -> Dispatch {
    return async(Queue.globalUserInteractive, closure: closure)
  }

  func asyncUserInteractive(closure: DispatchClosure) -> Dispatch {
    return async(Queue.globalUserInteractive, closure: closure)
  }

}

public extension Queue {
  static var globalUserInteractive: dispatch_queue_t { return global(Queue.Priority.userInteractive) }
  static var globalUserInitiated: dispatch_queue_t { return global(Queue.Priority.userInitiated) }
  static var globalUtility: dispatch_queue_t { return global(Queue.Priority.utility) }
  static var globalBackground: dispatch_queue_t { return global(Queue.Priority.background) }
}
